(ns rlm.qotd)
(rlm.rlm-commands/help)

;;;There is a pair of integers, 1 < m < n, whose sum is less
;;;than 100.

(def pos (fn [a]
           (filter (fn [[a b]] (< a b))
                   (map #(vector % a) (range 1 (- 100 a))))))

(def p0 (reduce concat (map pos (range 2 99))))

;;; Person S knows their sum, but nothing else about them.
;;; Person P knows their product, but nothing else about
;;; them.

;;; Now, Persons S and P know the above information, and each
;;; one knows that the other one knows it. They have the
;;; following conversation:

;;; P: I can't figure out what the numbers are.



;; Eliminate pairs with a unique product.

(defn group-by
  "Split coll into groups where the f maps each element in a
  group to the same value in O(n*log(n)) time."
  [f coll]
  (partition-by f (sort-by f coll)))

(defn unique-by
  "Remove all elements a,b of coll that for which
   (= (f a) (f b)) in O(n*log(n)) time."
  [f coll]
  (reduce
    concat
    (filter #(= (count %) 1) (group-by f coll))))
  
(defn multiple-by
  "Keep all elements a,b, a!=b of coll for which
   (= (f a) (f b)) in O(n*log(n)) time."
  [f coll]
  (reduce
    concat
    (filter #(> (count %) 1) (group-by f coll))))

(defn prod [[a b]] (* a b))

(def p1 (multiple-by prod p0))

;;; S: I was sure you couldn't.

;; Each possible sum s has a set of possible pairs [a b]
;; where (= s (+ a b)). Partition p0 (since he *was* sure)
;; by sum, and keep those pairs that belong in partitions
;; where each pair in that partition is in p1.

(defn sum [[a b]] (+ a b))

(def p2
  (reduce
   concat
   (filter
    (partial every? (set p1))
    (group-by sum p0))))


;;; P: Then I have.

;; Keep those pairs that have a unique product out of the
;; ones that are left.

(def p3
  (unique-by prod p2))


;;; S: Then I have, too.

